#include <iostream>
#include <vector>
#include <queue>
#include <set>

using namespace std;

struct Pozicija {
    int red, kolona;
};

int pronadjiPocetak(vector<vector<char>>& matrica, Pozicija& pocetak) {
    for (int i = 0; i < matrica.size(); ++i) {
        for (int j = 0; j < matrica[0].size(); ++j) {
            if (matrica[i][j] == 'S') {
                pocetak.red = i;
                pocetak.kolona = j;
                return 0;
            }
        }
    }
    return -1; // Pocetak nije pronadjen
}

int bfs(vector<vector<char>>& matrica, Pozicija pocetak) {
    vector<pair<int, int>> pravci = {{0, 1}, {0, -1}, {1, 0}, {-1, 0}};  // Desno, levo, dole, gore

    queue<pair<Pozicija, int>> red;  // Red sa pocetnim polozajem i brojacem koraka
    set<pair<int, int>> posecena;   // Set za pracenje posecenih polja

    red.push({pocetak, 0});

    while (!red.empty()) {
        Pozicija trenutnaPozicija = red.front().first;
        int koraci = red.front().second;
        red.pop();

        int redak = trenutnaPozicija.red;
        int kolona = trenutnaPozicija.kolona;

        if (matrica[redak][kolona] == 'C') {
            return koraci;
        }

        for (const auto& pravac : pravci) {
            int noviRed = redak + pravac.first;
            int novaKolona = kolona + pravac.second;

            if (noviRed >= 0 && noviRed < matrica.size() && novaKolona >= 0 && novaKolona < matrica[0].size() &&
                !posecena.count({noviRed, novaKolona})) {

                posecena.insert({noviRed, novaKolona});

                if (matrica[noviRed][novaKolona] != 'T') {
                    red.push({{noviRed, novaKolona}, koraci + 1});
                } else {
                    for (int i = 0; i < matrica.size(); ++i) {
                        for (int j = 0; j < matrica[0].size(); ++j) {
                            if (matrica[i][j] == 'T' && !(i == noviRed && j == novaKolona)) {
                                red.push({{i, j}, koraci + 1});
                            }
                        }
                    }
                }
            }
        }
    }

    return -1;  // Nije moguce stici do cilja
}

int main() {
    int R, K;
    cin >> R >> K;

    vector<vector<char>> matrica(R, vector<char>(K));

    for (int i = 0; i < R; ++i) {
        for (int j = 0; j < K; ++j) {
            cin >> matrica[i][j];
        }
    }

    Pozicija pocetak;
    if (pronadjiPocetak(matrica, pocetak) == -1) {
        cout << "Pocetak nije pronadjen." << endl;
        return 1;
    }

    int rezultat = bfs(matrica, pocetak);
    cout << rezultat << endl;

    return 0;
}