#include <iostream>
#include <vector>
#include <queue>
#include <set>

using namespace std;

bool validna_pozicija(int x, int y, int R, int K, const vector<vector<char>>& matrica) {
    return 0 <= x && x < R && 0 <= y && y < K && matrica[x][y] != 'T';
}

int bfs(pair<int, int> start, pair<int, int> cilj, const vector<vector<char>>& matrica, const vector<pair<int, int>>& kapije) {
    int R = matrica.size();
    int K = matrica[0].size();

    queue<pair<pair<int, int>, int>> red;
    red.push({start, 0});

    set<pair<int, int>> posjeceno;
    posjeceno.insert(start);

    while (!red.empty()) {
        auto trenutna_pozicija = red.front().first;
        int broj_koraka = red.front().second;
        red.pop();

        int x = trenutna_pozicija.first;
        int y = trenutna_pozicija.second;

        if (trenutna_pozicija == cilj) {
            return broj_koraka;
        }

        vector<pair<int, int>> pomeraji = {{1, 0}, {-1, 0}, {0, 1}, {0, -1}};

        for (const auto& pomeraj : pomeraji) {
            int nova_x = x + pomeraj.first;
            int nova_y = y + pomeraj.second;

            if (validna_pozicija(nova_x, nova_y, R, K, matrica) && !posjeceno.count({nova_x, nova_y})) {
                red.push({{nova_x, nova_y}, broj_koraka + 1});
                posjeceno.insert({nova_x, nova_y});
            }
        }

        for (const auto& kapija : kapije) {
            if (trenutna_pozicija != kapija) {
                red.push({kapija, broj_koraka + 1});
                posjeceno.insert(kapija);
            }
        }
    }

    return -1;
}

int najmanji_broj_koraka(const vector<vector<char>>& matrica) {
    int R = matrica.size();
    int K = matrica[0].size();

    pair<int, int> start, cilj;
    vector<pair<int, int>> kapije;

    for (int i = 0; i < R; ++i) {
        for (int j = 0; j < K; ++j) {
            if (matrica[i][j] == 'S') {
                start = {i, j};
            } else if (matrica[i][j] == 'C') {
                cilj = {i, j};
            } else if (matrica[i][j] == 'T') {
                kapije.push_back({i, j});
            }
        }
    }

    return bfs(start, cilj, matrica, kapije);
}

int main() {
    int R, K;
    cin >> R >> K;

    vector<vector<char>> matrica(R, vector<char>(K));

    for (int i = 0; i < R; ++i) {
        for (int j = 0; j < K; ++j) {
            cin >> matrica[i][j];
        }
    }

    int rezultat = najmanji_broj_koraka(matrica);

    cout << rezultat << endl;

    return 0;
}